@@ -1,5 +1,5 @@
 import math
-from inspect import getargspec
+from inspect import getfullargspec
 import warnings
 import matplotlib.pyplot as plt
 from matplotlib.patches import Circle, Rectangle, Wedge
@@ -9,7 +9,7 @@ import numpy as np
 from .system import System as _System
 from .system import _SingleDoFCoordinatesDict
 from .nonlinear_systems import MultiDoFNonLinearSystem as _MDNLS
-from .functions import benchmark_par_to_canonical
+from .functions import benchmark_par_to_canonical, spring, centered_rectangle
 
 
 class _LinearSystem(_System):
@@ -70,7 +70,7 @@ class _LinearSystem(_System):
     @canonical_coeffs_func.setter
     def canonical_coeffs_func(self, func):
         self._measurements._check_for_duplicate_keys()
-        for k in getargspec(func).args:
+        for k in getfullargspec(func).args:
             # NOTE : Measurements do not have to be time varying.
             if k not in (list(self.constants.keys()) +
                          list(self.measurements.keys())):
@@ -90,7 +90,7 @@ class _LinearSystem(_System):
             raise ValueError(msg)
         else:
             f = self.canonical_coeffs_func
-            args = [self._get_par_vals(k) for k in getargspec(f).args]
+            args = [self._get_par_vals(k) for k in getfullargspec(f).args]
             return f(*args)
 
 
@@ -697,7 +697,7 @@ class SingleDoFLinearSystem(_LinearSystem):
         axes[1].axvline(wn, color='black')
         axes[1].plot(w, np.rad2deg(phase_curve))
         axes[1].set_ylabel('Phase Shift [deg]')
-        axes[1].set_xlabel('Forcing Frequency, $\omega$, [rad/s]')
+        axes[1].set_xlabel(r'Forcing Frequency, $\omega$, [rad/s]')
 
         return axes
 
@@ -765,7 +765,7 @@ class MultiDoFLinearSystem(_MDNLS):
     @canonical_coeffs_func.setter
     def canonical_coeffs_func(self, func):
         self._measurements._check_for_duplicate_keys()
-        func_args = getargspec(func).args
+        func_args = getfullargspec(func).args
         for k in func_args:
             # NOTE : Measurements do not have to be time varying.
             if k not in (list(self.constants.keys()) +
@@ -827,8 +827,8 @@ class MultiDoFLinearSystem(_MDNLS):
         >>> sys.constants['g'] = 9.8  # m/s**2
         >>> sys.constants['l1'] = 1.0  # m
         >>> sys.constants['l2'] = 1.0  # m
-        >>> sys.constnats['m1'] = 0.5  # kg
-        >>> sys.constnats['m2'] = 0.5  # kg
+        >>> sys.constants['m1'] = 0.5  # kg
+        >>> sys.constants['m2'] = 0.5  # kg
         >>> sys.coordinates['theta1'] = 0.3  # rad
         >>> sys.coordinates['theta2'] = 0.0  # rad
         >>> sys.speeds['omega1'] = 0.0  # rad/s
@@ -856,7 +856,7 @@ class MultiDoFLinearSystem(_MDNLS):
 
     @forcing_func.setter
     def forcing_func(self, func):
-        args = [self._get_par_vals(k) for k in getargspec(func).args]
+        args = [self._get_par_vals(k) for k in getfullargspec(func).args]
         res = func(*args)
         msg = ("Forcing function must return the same number of values as "
                "the number of coordinates.")
@@ -870,7 +870,7 @@ class MultiDoFLinearSystem(_MDNLS):
                 raise ValueError(msg)
 
         self._forcing_func = func
-        self._forcing_func_arg_names = getargspec(func).args
+        self._forcing_func_arg_names = getfullargspec(func).args
 
     def canonical_coefficients(self):
         """Returns the mass, damping, and stiffness matrices in that order."""
@@ -880,7 +880,7 @@ class MultiDoFLinearSystem(_MDNLS):
             raise ValueError(msg)
         else:
             f = self.canonical_coeffs_func
-            args = [self._get_par_vals(k) for k in getargspec(f).args]
+            args = [self._get_par_vals(k) for k in getfullargspec(f).args]
             return f(*args)
 
     def _form_A_B(self):
@@ -1331,6 +1331,30 @@ class MassSpringDamperSystem(SingleDoFLinearSystem):
 
         self.canonical_coeffs_func = coeffs
 
+        block_size = 1.0
+
+        def plot_config(position):
+            fig, ax = plt.subplots(1, 1)
+
+            block = Rectangle(*centered_rectangle(position, 0.0, block_size,
+                                                  block_size))
+            ax.add_patch(block)
+            spring_line = ax.plot(*spring(0.0, position, 0.0, 0.0, 0.25, n=10), 'k')[0]
+
+            ax.set_xlim((-position - block_size / 2, position + block_size / 2))
+            ax.set_aspect('equal')
+
+            return fig, block, spring_line
+
+        self.config_plot_func = plot_config
+
+        def update(position, block, spring_line):
+            block.set_xy(centered_rectangle(position, 0.0, block_size,
+                                            block_size)[0])
+            spring_line.set_data(*spring(0.0, position, 0.0, 0.0, 0.25, n=10))
+
+        self.config_plot_update_func = update
+
 
 class BaseExcitationSystem(SingleDoFLinearSystem):
     """This system represents a mass connected to a moving massless base via a
@@ -1625,9 +1649,9 @@ class SimpleQuarterCarSystem(BaseExcitationSystem):
             # results available
             road = ax.plot(lat, np.zeros_like(lat), color='black')[0]
 
-            suspension = ax.plot([lat_pos, lat_pos],
-                                 [0.0, xeq + car_vertical_position],
-                                 linewidth='4', marker='o', color='yellow')[0]
+            suspension = ax.plot(*spring(lat_pos, lat_pos,
+                                 0.0, xeq + car_vertical_position, 0.1, n=2),
+                                 linewidth='2', marker='o', color='yellow')[0]
             #force_vec = ax.plot([lat_pos, lat_pos],
                                 #[xeq + car_vertical_position + rect_height / 2,
                                  #xeq + car_vertical_position + rect_height / 2 + 0.2],
@@ -1657,8 +1681,8 @@ class SimpleQuarterCarSystem(BaseExcitationSystem):
             road.set_xdata(lat)
             road.set_ydata(np.hstack((road_height__hist, road_height__futr)))
 
-            suspension.set_xdata([lat_pos, lat_pos])
-            suspension.set_ydata([road_height, xeq + car_vertical_position])
+            suspension.set_data(*spring(lat_pos, lat_pos, road_height,
+                                        xeq + car_vertical_position, 0.1, n=2))
 
             #force_vec.set_xdata([lat_pos, lat_pos])
             #force_vec.set_ydata([xeq + x[i] + rect_height / 2,
@@ -1899,3 +1923,201 @@ class BicycleSystem(MultiDoFLinearSystem):
             return M, v * C1, g * K0 + v**2 * K2
 
         self.canonical_coeffs_func = can_coeff_matrices
+
+
+class AutomobileLateralSystem(MultiDoFLinearSystem):
+
+    def __init__(self):
+
+        super(AutomobileLateralSystem, self).__init__()
+
+        speed = 20.0  # m/s
+
+        car_width = 1.54  # meters
+        car_length = 2.7  # meters
+
+        tire_width = 0.22 # meters
+        tire_diameter = 0.65  # meters
+
+        mass = 2760 * 4.44 / 9.81  # kg (2760 lbs converted to kg)
+        inertia = mass / 12 * (car_length**2 + car_width**2)  # kg m**2
+
+        com_ratio = 0.2  # < 0.5 closer to front, > 0.5 closer to rear
+        front_end = com_ratio * car_length  # meters
+        rear_end = (1 - com_ratio) * car_length  # meters
+
+        dmux_dalpha = 0.6 / np.deg2rad(5.0)  # (N/N) / rad
+        Fz = mass * 9.81  # [N] weight of vehicle
+
+        front_corner_coef = 0.6 * dmux_dalpha * Fz  # N/rad
+        rear_corner_coef = 0.4 * dmux_dalpha * Fz  # N/rad
+
+        self.constants['U'] = speed
+        self.constants['m'] = mass
+        self.constants['a'] = front_end
+        self.constants['b'] = rear_end
+        self.constants['I'] = inertia
+        self.constants['Cf'] = front_corner_coef
+        self.constants['Cr'] = rear_corner_coef
+
+        self.coordinates['y'] = 0.0
+        self.coordinates['psi'] = 0.0
+
+        self.speeds['ydot'] = 0.0
+        self.speeds['psidot'] = 0.0
+
+        def can_coeffs_func(U, m, I, a, b, Cf, Cr):
+
+            M = np.diag([m, I])
+
+            C = np.array([[(Cf + Cr) / U,     (a*Cf - b*Cr) / U],
+                          [(a*Cf - b*Cr) / U, (a**2*Cf + b**2*Cr) / U]])
+
+            K = np.array([[0, -(Cf + Cr)],
+                          [0, -(a*Cf - b*Cr)]])
+
+            return M, C, K
+
+        self.canonical_coeffs_func = can_coeffs_func
+
+        def front_steer(time):
+            return np.deg2rad(8.0) * np.cos(np.deg2rad(180/8) * time)
+
+        self.add_measurement('deltaf', front_steer)
+
+        def rear_steer(deltaf):
+            return 0.0 * deltaf
+
+        self.add_measurement('deltar', rear_steer)
+
+        def forcing_func(a, b, Cf, Cr, deltaf, deltar):
+            return Cf * deltaf + Cr * deltar, a * Cf * deltaf - b * Cr * deltar
+
+        self.forcing_func = forcing_func
+
+        def plot_config(y, psi, a, b, U, deltaf, deltar, time, time__hist,
+                        y__hist):
+
+            fig, ax = plt.subplots(1, 1)
+
+            car_com_x = U * time
+            car_com_y = y
+
+            com = Circle((car_com_x, car_com_y), radius=0.1, color='black')
+
+            car_center_x = car_com_x - ((a + b) / 2 - a) * np.cos(psi)
+            car_center_y = car_com_y - ((a + b) / 2 - a) * np.sin(psi)
+
+            car = Rectangle(*centered_rectangle((car_center_x, car_center_y),
+                                                a + b,
+                                                car_width,
+                                                angle=np.rad2deg(psi)))
+
+            flwheel = Rectangle(*centered_rectangle(
+                (car_com_x + a*np.cos(psi) + car_width / 2 * np.sin(psi),
+                car_com_y + a*np.sin(psi) - car_width / 2 * np.cos(psi)),
+                tire_diameter, tire_width, angle=np.rad2deg(psi + deltaf)),
+                color='green')
+            frwheel = Rectangle(*centered_rectangle(
+                (car_com_x + a*np.cos(psi) - car_width / 2 * np.sin(psi),
+                car_com_y + a*np.sin(psi) + car_width / 2 * np.cos(psi)),
+                tire_diameter, tire_width, angle=np.rad2deg(psi + deltaf)),
+                color='red')
+            rlwheel = Rectangle(*centered_rectangle(
+                (car_com_x - b*np.cos(psi) + car_width / 2 * np.sin(psi),
+                car_com_y - b*np.sin(psi) - car_width / 2 * np.cos(psi)),
+                tire_diameter, tire_width, angle=np.rad2deg(psi + deltar)),
+                color='orange')
+            rrwheel = Rectangle(*centered_rectangle(
+                (car_com_x - b*np.cos(psi) - car_width / 2 * np.sin(psi),
+                car_com_y - b*np.sin(psi) + car_width / 2 * np.cos(psi)),
+                tire_diameter, tire_width, angle=np.rad2deg(psi + deltar)),
+                color='purple')
+
+            path = ax.plot(U * time__hist, y__hist, color='black')[0]
+
+            ax.add_patch(car)
+            ax.add_patch(com)
+            ax.add_patch(flwheel)
+            ax.add_patch(frwheel)
+            ax.add_patch(rlwheel)
+            ax.add_patch(rrwheel)
+
+            text = ax.text(U * time - 3*car_length, y + 3*car_width,
+                           'Time = {:0.3f} s'.format(time))
+
+            ax.set_xlim((U * time - 4*(a+b), U * time + 4*(a+b)))
+            ax.set_ylim((y + 4*car_width, y - 4*car_width))
+
+            ax.set_aspect('equal')
+            ax.grid(b=True)
+            ax.set_xlabel('Distance [m]')
+            ax.set_ylabel('Distance [m]')
+
+            return (fig, ax, car, com, flwheel, frwheel, rlwheel, rrwheel,
+                    path, text)
+
+
+        self.config_plot_func = plot_config
+
+        def update(y, psi, a, b, U, deltaf, deltar, time, time__hist, y__hist,
+                   ax, car, com, flwheel, frwheel, rlwheel, rrwheel, path,
+                   text):
+
+            car_com_x = U * time
+            car_com_y = y
+
+            com.center = (car_com_x, car_com_y)
+
+            car_center_x = car_com_x - ((a + b) / 2 - a) * np.cos(psi)
+            car_center_y = car_com_y - ((a + b) / 2 - a) * np.sin(psi)
+            car_center = (car_center_x, car_center_y)
+
+            xy, _, _, angle = centered_rectangle(car_center, car_length,
+                                                car_width, np.rad2deg(psi))
+            car.set_xy(xy)
+            car.angle = angle
+
+            res = centered_rectangle(
+                (car_com_x + a*np.cos(psi) + car_width / 2 * np.sin(psi),
+                car_com_y + a*np.sin(psi) - car_width / 2 * np.cos(psi)),
+                tire_diameter, tire_width, angle=np.rad2deg(psi + deltaf))
+
+            flwheel.set_xy(res[0])
+            flwheel.angle = res[-1]
+
+            res = centered_rectangle(
+                (car_com_x + a*np.cos(psi) - car_width / 2 * np.sin(psi),
+                car_com_y + a*np.sin(psi) + car_width / 2 * np.cos(psi)),
+                tire_diameter, tire_width, angle=np.rad2deg(psi + deltaf))
+
+            frwheel.set_xy(res[0])
+            frwheel.angle = res[-1]
+
+            res = centered_rectangle(
+                (car_com_x - b*np.cos(psi) + car_width / 2 * np.sin(psi),
+                car_com_y - b*np.sin(psi) - car_width / 2 * np.cos(psi)),
+                tire_diameter, tire_width, angle=np.rad2deg(psi + deltar))
+
+            rlwheel.set_xy(res[0])
+            rlwheel.angle = res[-1]
+
+            res = centered_rectangle(
+                (car_com_x - b*np.cos(psi) - car_width / 2 * np.sin(psi),
+                car_com_y - b*np.sin(psi) + car_width / 2 * np.cos(psi)),
+                tire_diameter, tire_width, angle=np.rad2deg(psi + deltar))
+
+            rrwheel.set_xy(res[0])
+            rrwheel.angle = res[-1]
+
+            path.set_data(U * time__hist, y__hist)
+
+            text.set_text('Time = {:0.3f} s'.format(time))
+            text.set_position((U * time - 3*car_length, y + 3*car_width))
+
+            ax.set_xlim((U * time - 4*(a+b), U * time + 4*(a+b)))
+            ax.set_ylim((y + 4*car_width, y - 4*car_width))
+
+            ax.set_aspect('equal')
+
+        self.config_plot_update_func = update
